/**
  This file is part of InfoVisTk

  Copyright (c) 2011 Bertjan Broeksema <broeksema@kde.org>

  InfoVisTk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/


#include "benchmark.h"

#include <QtTest/QTest>

#include "dpointerclass.h"
#include "valueclass.h"
#include "inlinevalueclass.h"

Benchmark::Benchmark(QObject *parent) :
    QObject(parent)
{
}

void Benchmark::testDPointerCreateDestruct()
{
  QBENCHMARK {
    DPointerClass object( 4 );
  }
}

void Benchmark::testDPointerHeapCreateDestruct()
{
  QBENCHMARK {
    DPointerClass *object = new DPointerClass( 4 );
    delete object;
  }
}

void Benchmark::testDPointerCopy()
{
  DPointerClass object( 4 );
  QBENCHMARK {
    DPointerClass object2( object );
  }
}

void Benchmark::testDPointerAssignment()
{
  DPointerClass object1( 4 );
  DPointerClass object2( 2 );

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), 2 );

  QBENCHMARK {
    object2 = object1;
  }

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), object1.value() );
}

void Benchmark::testDPointerConstAccess()
{
  DPointerClass object( 4 );
  QBENCHMARK {
    object.value();
  }
}

void Benchmark::testDPointerNonConstAccess()
{
  DPointerClass object( 4 );
  int i = 0;
  QBENCHMARK {
    object.setValue( ++i );
  }
}

/// Value

void Benchmark::testValueCreateDestruct()
{
  QBENCHMARK {
    ValueClass object( 4 );
  }
}

void Benchmark::testValueHeapCreateDestruct()
{
  QBENCHMARK {
    ValueClass *object = new ValueClass( 4 );
    delete object;
  }
}

void Benchmark::testValueCopy()
{
  ValueClass object( 4 );
  QBENCHMARK {
    ValueClass object2( object );
  }
}

void Benchmark::testValueAssignment()
{
  ValueClass object1( 4 );
  ValueClass object2( 2 );

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), 2 );

  QBENCHMARK {
    object2 = object1;
  }

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), object1.value() );
}

void Benchmark::testValueConstAccess()
{
  ValueClass object( 4 );
  QBENCHMARK {
    object.value();
  }
}

void Benchmark::testValueNonConstAccess()
{
  ValueClass object( 4 );
  int i = 0;
  QBENCHMARK {
    object.setValue( ++i );
  }
}

/// Inline value

void Benchmark::testInlineValueCreateDestruct()
{
  QBENCHMARK {
    InlineValueClass object( 4 );
  }
}

void Benchmark::testInlineValueHeapCreateDestruct()
{
  QBENCHMARK {
    InlineValueClass *object = new InlineValueClass( 4 );
    delete object;
  }
}

void Benchmark::testInlineValueCopy()
{
  InlineValueClass object( 4 );
  QBENCHMARK {
    InlineValueClass object2( object );
  }
}

void Benchmark::testInlineValueAssignment()
{
  InlineValueClass object1( 4 );
  InlineValueClass object2( 2 );

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), 2 );

  QBENCHMARK {
    object2 = object1;
  }

  QCOMPARE( object1.value(), 4 );
  QCOMPARE( object2.value(), object1.value() );
}

void Benchmark::testInlineValueConstAccess()
{
  InlineValueClass object( 4 );
  QBENCHMARK {
    object.value();
  }
}

void Benchmark::testInlineValueNonConstAccess()
{
  InlineValueClass object( 4 );
  int i = 0;
  QBENCHMARK {
    object.setValue( ++i );
  }
}

QTEST_MAIN( Benchmark )
